@@ -75,6 +75,30 @@ int scsi_init_command(struct command *c, unsigned char *opcode, int oplen)
 }
 
 
+/* ts B00728 */
+int spc_decode_sense(unsigned char *sense, int senselen,
+                     int *key, int *asc, int *ascq)
+{
+	*key = *asc = *ascq = 0;
+	if ((sense[0] & 0x7f) == 0x72 || (sense[0] & 0x7f) == 0x73) {
+		if (senselen <= 0 || senselen > 1)
+			*key = sense[1] & 0x0f;
+		if (senselen <= 0 || senselen > 2)
+			*asc = sense[2];
+		if (senselen <= 0 || senselen > 3)
+			*ascq = sense[3];
+		return 1;
+	}
+	if (senselen <= 0 || senselen > 2)
+		*key = sense[2] & 0x0f;
+	if (senselen <= 0 || senselen > 12)
+		*asc = sense[12];
+	if (senselen <= 0 || senselen > 13)
+		*ascq = sense[13];
+	return 1;
+}
+
+
 int spc_test_unit_ready_r(struct burn_drive *d, int *key, int *asc, int *ascq)
 {
 	struct command c;
@@ -83,23 +107,23 @@ int spc_test_unit_ready_r(struct burn_drive *d, int *key, int *asc, int *ascq)
 		return 0;
 
 	scsi_init_command(&c, SPC_TEST_UNIT_READY,sizeof(SPC_TEST_UNIT_READY));
-/*
-	c.oplen = sizeof(SPC_TEST_UNIT_READY);
-	memcpy(c.opcode, SPC_TEST_UNIT_READY, sizeof(SPC_TEST_UNIT_READY));
-	c.page = NULL;
-*/
 	c.retry = 0;
 	c.dir = NO_TRANSFER;
 	d->issue_command(d, &c);
 	if (c.error) {
-		*key= c.sense[2];
-		*asc= c.sense[12];
-		*ascq= c.sense[13];
-		return (c.sense[2] & 0xF) == 0;
+
+/*
+fprintf(stderr, "sense[0 - 2] = %2.2X %2.2X %2.2X",
+c.sense[0], c.sense[1], c.sense[2]);
+*/
+
+		spc_decode_sense(c.sense, 0, key, asc, ascq);
+		return (key == 0);
 	}
 	return 1;
 }
 
+
 int spc_test_unit_ready(struct burn_drive *d)
 {
 	int key,asc,ascq;
@@ -151,6 +175,7 @@ handle_error:;
 			/* ts A90213 */
 			sprintf(msg,
 				"Asynchronous SCSI error on %s: ", cmd_text);
+			sense[0] = 0x70; /* Fixed format sense data */
 			sense[2] = key;
 			sense[12] = asc;
 			sense[13] = ascq;
@@ -803,9 +828,7 @@ void spc_probe_write_modes(struct burn_drive *d)
 		if (last_try)
 	break;
 
-		key = c.sense[2];
-		asc = c.sense[12];
-		ascq = c.sense[13];
+		spc_decode_sense(c.sense, 0, &key, &asc, &ascq);
 		if (key)
 			burn_print(7, "%d not supported\n", try_block_type);
 		else {
@@ -958,6 +981,7 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 			     int senselen, char msg_data[161],
 			     int *key, int *asc, int *ascq)
 {
+	int ret;
 	char *msg;
 	static char key_def[16][40] = {
 		"(no specific error)",
@@ -981,12 +1005,9 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 	msg= msg_data;
 	*key= *asc= *ascq= -1;
 
-	if (senselen<=0 || senselen>2)
-		*key = sense[2] & 0x0f;
-	if (senselen<=0 || senselen>12)
-		*asc = sense[12];
-	if (senselen<=0 || senselen>13)
-		*ascq = sense[13];
+	ret = spc_decode_sense(sense, senselen, key, asc, ascq);
+	if (ret <= 0)
+		*key= *asc= *ascq= -1;
 
 	sprintf(msg, "[%X %2.2X %2.2X] ", *key, *asc, *ascq);
 	msg= msg + strlen(msg);
@@ -996,8 +1017,10 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 
 	switch (*asc) {
 	case 0x00:
+		if (*key > 0 || *ascq > 0) 
+			break; /* Fall through to unknown error */
 		sprintf(msg, "(No error reported by SCSI transaction)");
-		return RETRY;
+		return GO_ON;
 
 	case 0x02:
 		sprintf(msg, "Not ready");
@@ -1203,6 +1226,12 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 			sprintf(msg, "Medium not present");
 		d->status = BURN_DISC_EMPTY;
 		return FAIL;
+	case 0x3E:
+		if (*ascq == 1)
+			sprintf(msg, "Logical unit failure");
+		else if (*ascq == 2)
+			sprintf(msg, "Timeout on logical unit");
+		return FAIL;
 	case 0x57:
 		if (*key != 3 || *ascq != 0)
 			break;
@@ -1379,28 +1408,10 @@ int scsi_notify_error(struct burn_drive *d, struct command *c,
 	sprintf(msg, "SCSI error condition on command %2.2Xh %s: ",
 		c->opcode[0],
 		scsi_command_name((unsigned int) c->opcode[0], 0));
-
-#ifdef NIX
-	if (key>=0)
-		sprintf(msg+strlen(msg), " key=%Xh", key);
-	if (asc>=0)
-		sprintf(msg+strlen(msg), " asc=%2.2Xh", asc);
-	if (ascq>=0)
-		sprintf(msg+strlen(msg), " ascq=%2.2Xh", ascq);
-	ret = libdax_msgs_submit(libdax_messenger, d->global_index, 0x0002010f,
-			LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH, msg,0,0);
-	if (ret < 0)
-		return ret;
-	ret = libdax_msgs_submit(libdax_messenger, d->global_index, 0x0002010f,
-			LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
-			scsi_msg,0,0);
-#else
 	strcat(msg, scsi_msg);
 	ret = libdax_msgs_submit(libdax_messenger, d->global_index, 0x0002010f,
 			LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH, msg,0,0);
 
-#endif /* NIX */
-
 	return ret;
 }
 
@@ -1493,19 +1504,31 @@ int scsi_log_cmd(struct command *c, void *fp_in, int flag)
                  bit1 do not print duration
 */
 int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
-		 int duration, int flag)
+		 int sense_len, int duration, int flag)
 {
 	char durtxt[20];
 	FILE *fp = fp_in;
+	int key, asc, ascq, i, l;
 
-      	if(fp != NULL && (fp == stderr || (burn_sg_log_scsi & 1))) {
-		if(flag & 1) {
+      	if (fp != NULL && (fp == stderr || (burn_sg_log_scsi & 1))) {
+		if (flag & 1) {
+			l = 18;
+			if ((sense[0] & 0x7f) == 0x72 ||
+			    (sense[0] & 0x7f) == 0x73)
+				l = sense[7] + 7 + 1; /* SPC-5 4.5.2. */
+			if (l > sense_len)
+				l = sense_len;
+			fprintf(fp, "+++ sense data =");
+			for (i = 0 ; i < l; i++)
+				fprintf(fp, " %2.2X", sense[i]);
+			fprintf(fp, "\n");
 			durtxt[0] = 0;
 			if (!(flag & 2))
-  				sprintf(durtxt, "   (%6d ms)\n",duration);
+  				sprintf(durtxt, "   (%6d ms)", duration);
+			spc_decode_sense(sense, 0, &key, &asc, &ascq);
   			fprintf(fp, "+++ key=%X  asc=%2.2Xh  ascq=%2.2Xh%s\n",
-				sense[2], sense[12], sense[13], durtxt);
-
+				(unsigned int) key, (unsigned int) asc,
+				(unsigned int) ascq, durtxt);
 		} else {
 			scsi_show_cmd_reply(c, fp, 0);
 			if (!(flag & 2))
@@ -1516,7 +1539,55 @@ int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
 	}
 	if (fp == stderr || !(burn_sg_log_scsi & 2))
 		return 1;
-	scsi_log_err(c, stderr, sense, duration, flag);
+	scsi_log_err(c, stderr, sense, sense_len, duration, flag);
 	return 1;
 }
 
+
+/* ts B00808 */
+/*
+    @param flag    bit0 = do not retry
+                   bit1 = do not print duration
+    @return 0 = not yet done , 1 = done , -1 = error
+*/
+int scsi_eval_cmd_outcome(struct burn_drive *d, struct command *c, void *fp,
+			unsigned char *sense, int sense_len,
+			int duration, time_t start_time, int timeout_ms,
+			int loop_count, int flag)
+{
+	enum response outcome;
+	int done = -1, usleep_time;
+
+	if (sense_len <= 0)
+		return 1;
+		
+	if (burn_sg_log_scsi & 3)
+		scsi_log_err(c, fp, sense, sense_len, duration,
+				 1 | (flag & 2));
+	outcome = scsi_error(d, sense, sense_len);
+	if (outcome == RETRY && c->retry && !(flag & 1)) {
+		/* Calming down retries and breaking up endless cycle
+		*/
+		usleep_time = Libburn_scsi_retry_usleeP +
+				loop_count * Libburn_scsi_retry_incR;
+		if (time(NULL) + usleep_time / 1000000 - start_time >
+		    timeout_ms / 1000 + 1) {
+			done = 1;
+			goto ex;
+		}
+		usleep(usleep_time);
+		if (burn_sg_log_scsi & 3) 
+			scsi_log_cmd(c, fp, 0);
+		return 0;
+	} else if (outcome == RETRY) {
+		done = 1;
+	} else if (outcome == GO_ON) {
+		return 1;
+	} else if (outcome == FAIL) {
+		done = 1;
+	}
+ex:;
+	c->error = 1;
+	scsi_notify_error(d, c, sense, sense_len, 0);
+	return done;
+}
