@@ -2006,7 +2006,13 @@ ex:;
 	burn_drive_mark_unready(d);
 	burn_drive_inquire_media(d);
 
-	/* <<< d->busy = BURN_DRIVE_IDLE; */
+	if (d->current_profile == 0x41 && d->complete_sessions >= 300) {
+		sprintf(msg, "Sequential BD-R media now contains %d sessions. It is likely to soon fail writing.", d->complete_sessions);
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+				0x0002017b, LIBDAX_MSGS_SEV_WARNING,
+				LIBDAX_MSGS_PRIO_ZERO, msg, 0, 0);
+	}
+
 	return ret;
 early_failure:;
 	return 0;
@@ -2114,9 +2120,11 @@ int burn_stdio_mmc_write(struct burn_drive *d, int start, struct buffer *buf)
 	if (d->cancel)
 		return BE_CANCELLED;
 	if (d->stdio_fd < 0) {
-
-		/* >>> program error */;
-
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			0x0002017d,
+			LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
+			"Invalid file descriptor with stdio pseudo-drive",
+			0, 0);
 		d->cancel = 1;
 		return BE_CANCELLED;
 	}
@@ -2164,9 +2172,11 @@ int burn_stdio_mmc_dummy_write(struct burn_drive *d, int start,
 int burn_stdio_sync_cache(int fd, struct burn_drive *d, int flag)
 {
 	if (fd < 0) {
-
-		/* >>> program error */;
-
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			0x0002017d,
+			LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
+			"Invalid file descriptor with stdio pseudo-drive",
+			0, 0);
 		d->cancel = 1;
 		return 0;
 	}
