@@ -545,6 +545,20 @@ struct isoburn_imgen_opts {
     /* 1 to 255, 0= disabled/default */
     int partition_heads_per_cyl;
 
+    /* Parameters and state of Jigdo Template Export environment.
+    */
+    void *libjte_handle;
+
+    /* A trailing padding of zero bytes which belongs to the image
+    */
+    uint32_t tail_blocks;
+
+    /* Eventual disk file paths of prepared images which shall be appended
+       after the ISO image and described by partiton table entries in a MBR.
+    */
+    char *appended_partitions[4];
+    uint8_t appended_part_types[4];
+
 };
 
 
