@@ -224,7 +224,7 @@ void isoburn_version(int *major, int *minor, int *micro);
 */
 #define isoburn_libisofs_req_major  0
 #define isoburn_libisofs_req_minor  6
-#define isoburn_libisofs_req_micro 36 
+#define isoburn_libisofs_req_micro 38 
 
 /** The minimum version of libburn to be used with this version of libisoburn
     at compile time.
@@ -232,8 +232,16 @@ void isoburn_version(int *major, int *minor, int *micro);
 */
 #define isoburn_libburn_req_major  0
 #define isoburn_libburn_req_minor  8
-#define isoburn_libburn_req_micro  4
+#define isoburn_libburn_req_micro  8
 
+/** The minimum compile time requirements of libisoburn towards libjte are
+    the same as of a suitable libisofs towards libjte.
+    So use these macros from libisofs.h :
+      iso_libjte_req_major
+      iso_libjte_req_minor
+      iso_libjte_req_micro
+    @since 0.6.4
+*/
 
 /** The minimum version of libisofs to be used with this version of libisoburn
     at runtime. This is checked already in isoburn_initialize() which will
@@ -248,6 +256,17 @@ void isoburn_version(int *major, int *minor, int *micro);
 int isoburn_libisofs_req(int *major, int *minor, int *micro);
 
 
+/** The minimum version of libjte to be used with this version of libisoburn
+    at runtime. The use of libjte is optional and depends on configure
+    tests. It can be prevented by ./configure option --disable-libjte .
+    This is checked already in isoburn_initialize() which will refuse on
+    outdated version. So this call is for information purposes after
+    successful startup only.
+    @since 0.6.4
+*/
+int isoburn_libjte_req(int *major, int *minor, int *micro);
+
+
 /** The minimum version of libburn to be used with this version of libisoburn
     at runtime. This is checked already in isoburn_initialize() which will
     refuse on outdated version. So this call is for information purposes after
@@ -268,7 +287,7 @@ int isoburn_libburn_req(int *major, int *minor, int *micro);
 */
 #define isoburn_header_version_major  0
 #define isoburn_header_version_minor  6
-#define isoburn_header_version_micro  2
+#define isoburn_header_version_micro  4
 /** Note:
     Above version numbers are also recorded in configure.ac because libtool
     wants them as parameters at build time.
@@ -1215,6 +1234,16 @@ int isoburn_igopt_get_scdbackup_tag(struct isoburn_imgen_opts *o,
                           SYSLINUX plus an ISOLINUX boot image (see
                           iso_image_set_boot_image()) and only if not bit0
                           is set.
+                    bit2-7= System area type
+                          0= with bit0 or bit1: MBR
+                             else: unspecified type 
+                          @since 0.6.4
+                          1= MIPS Big Endian Volume Header
+                   >>> EXPERIMENTAL:
+                   Submit MIPS boot image files as El Torito Boot image to
+                   iso_image_set_boot_image() , iso_image_add_boot_image().
+                   No El Torito info will be produced with system area type 1.
+
     @return 1 success, 0 no data to get, <0 failure
 */
 int isoburn_igopt_set_system_area(struct isoburn_imgen_opts *o,
@@ -1295,6 +1324,105 @@ int isoburn_igopt_get_pvd_times(struct isoburn_imgen_opts *opts,
                       char uuid[17]);
 
 
+/** Associate a libjte environment object to the upcomming write run.
+    libjte implements Jigdo Template Extraction as of Steve McIntyre and
+    Richard Atterer.
+    A non-NULL libjte_handle will cause failure to write if libjte was not
+    enabled in libisofs at compile time.
+    @since 0.6.4
+    @param opts
+           The option set to work on
+    @param libjte_handle
+           Pointer to a struct libjte_env e.g. created by libjte_new().
+           It must stay existent from the start of image writing by
+           isoburn_prepare_*() until the write thread has ended.
+           E.g. until libburn indicates the end of its write run.
+    @return 1 success, <=0 failure
+*/
+int isoburn_igopt_attach_jte(struct isoburn_imgen_opts *opts,
+                             void *libjte_handle);
+
+/** Remove eventual association to a libjte environment handle.
+    @since 0.6.4
+    @param opts
+           The option set to work on
+    @param libjte_handle
+           If not submitted as NULL, this will return the previously set
+           libjte handle. 
+    @return 1 success, <=0 failure
+*/
+int isoburn_igopt_detach_jte(struct isoburn_imgen_opts *opts,
+                             void **libjte_handle);
+
+
+/** Set resp. get the number of trailing zero byte blocks to be written by
+    libisofs. The image size counter of the emerging ISO image will include
+    them. Eventual checksums will take them into respect.
+    They will be written immediately before the eventual image checksum area
+    which is at the very end of the image.
+    For a motivation see iso_write_opts_set_tail_blocks() in libisofs.h .
+    @since 0.6.4
+    @param opts
+           The option set to work on
+    @aram num_blocks
+           Number of extra 2 kB blocks to be written by libisofs.
+    @return 1 success, <=0 failure
+*/
+int isoburn_igopt_set_tail_blocks(struct isoburn_imgen_opts *opts,
+                                  uint32_t num_blocks);
+int isoburn_igopt_get_tail_blocks(struct isoburn_imgen_opts *opts,
+                                  uint32_t *num_blocks);
+
+
+/** Cause an arbitrary data file to be appended to the ISO image and to be
+    described by a partition table entry in an MBR at the start of the
+    ISO image.
+    The partition entry will bear the size of the image file rounded up to
+    the next multiple of 2048 bytes.
+    @since 0.6.4
+    @param opts
+           The option set to be manipulated.
+    @param partition_number
+           Depicts the partition table entry which shall describe the
+           appended image. Range 1 to 4.
+           1 will cause the whole ISO image to be unclaimable space before
+           partition 1.
+    @param image_path
+           File address in the local file system.
+    @param image_type
+           The partition type. E.g. FAT12 = 0x01 , FAT16 = 0x06,
+           Linux Native Partition = 0x83. See fdisk command L.
+    @return
+           <=0 = error, 1 = success
+*/
+int isoburn_igopt_set_partition_img(struct isoburn_imgen_opts *opts,
+                                  int partition_number, uint8_t partition_type,
+                                  char *image_path);
+
+/** Inquire the current settings made by isoburn_igopt_set_partition_img().
+    @since 0.6.4
+    @param opts
+           The option set to be inquired.
+    @param num_entries
+           Number of array elements in partition_types[] and image_paths[].
+    @param partition_types
+           The partition type associated with the partition. Valid only if
+           image_paths[] of the same index is not NULL.
+    @param image_paths
+           Its elements get filled with either NULL or a pointer to a string
+           with a file address,
+    @return
+           <0 = error
+            0 = no partition image set
+           >0 highest used partition number
+*/
+int isoburn_igopt_get_partition_img(struct isoburn_imgen_opts *opts,
+                                    int num_entries,
+                                    uint8_t partition_types[],
+                                    char *image_paths[]);
+
+
+
 /* ----------------------------------------------------------------------- */
 /*                      End of Options for image generation                */
 /* ----------------------------------------------------------------------- */
