@@ -41,6 +41,20 @@
 #include <langinfo.h>
 #include <stdio.h>
 
+#ifdef Xorriso_standalonE
+
+#ifdef Xorriso_with_libjtE
+#include "../libjte/libjte.h"
+#endif
+
+#else
+
+#ifdef Libisofs_with_libjtE
+#include <libjte/libjte.h>
+#endif
+
+#endif /* ! Xorriso_standalonE */
+
 /*
  * TODO #00011 : guard against bad path table usage with more than 65535 dirs
  * image with more than 65535 directories have path_table related problems
@@ -55,6 +69,12 @@ void ecma119_image_free(Ecma119Image *t)
 
     if (t == NULL)
         return;
+
+    if (t->refcount > 1) {
+        t->refcount--;
+        return;
+    }
+
     if (t->root != NULL)
         ecma119_node_free(t->root);
     if (t->image != NULL)
@@ -87,10 +107,34 @@ void ecma119_image_free(Ecma119Image *t)
         free(t->writers);
     if (t->partition_root != NULL)
         ecma119_node_free(t->partition_root);
-    t->partition_root = NULL;
+    for (i = 0; i < 4; i++)
+        if (t->appended_partitions[i] != NULL)
+            free(t->appended_partitions[i]);
+
     free(t);
 }
 
+static int show_chunk_to_jte(Ecma119Image *target, char *buf, int count)
+{
+
+#ifdef Libisofs_with_libjtE
+
+    int ret;
+
+    if (target->libjte_handle == NULL)
+        return ISO_SUCCESS;
+    ret = libjte_show_data_chunk(target->libjte_handle, buf, count, 1); 
+    if (ret <= 0) {
+        iso_libjte_forward_msgs(target->libjte_handle,
+                                target->image->id, ISO_LIBJTE_FILE_FAILED, 0);
+        return ISO_LIBJTE_FILE_FAILED;
+    }
+
+#endif /* Libisofs_with_libjtE */
+
+    return ISO_SUCCESS;
+}
+
 /**
  * Check if we should add version number ";" to the given node name.
  */
@@ -896,7 +940,7 @@ int ecma119_writer_create(Ecma119Image *target)
 
 /** compute how many padding bytes are needed */
 static
-int pad_writer_compute_data_blocks(IsoImageWriter *writer)
+int mspad_writer_compute_data_blocks(IsoImageWriter *writer)
 {
     Ecma119Image *target;
     uint32_t min_size;
@@ -907,24 +951,25 @@ int pad_writer_compute_data_blocks(IsoImageWriter *writer)
     target = writer->target;
     min_size = 32 + target->partition_offset;
     if (target->curblock < min_size) {
-        target->pad_blocks = min_size - target->curblock;
+        target->mspad_blocks = min_size - target->curblock;
         target->curblock = min_size;
     }
     return ISO_SUCCESS;
 }
 
 static
-int pad_writer_write_vol_desc(IsoImageWriter *writer)
+int mspad_writer_write_vol_desc(IsoImageWriter *writer)
 {
     /* nothing to do */
     return ISO_SUCCESS;
 }
+
 static
-int pad_writer_write_data(IsoImageWriter *writer)
+int mspad_writer_write_data(IsoImageWriter *writer)
 {
     int ret;
     Ecma119Image *t;
-    uint32_t pad[BLOCK_SIZE];
+    uint8_t pad[BLOCK_SIZE];
     size_t i;
 
     if (writer == NULL) {
@@ -932,12 +977,12 @@ int pad_writer_write_data(IsoImageWriter *writer)
     }
     t = writer->target;
 
-    if (t->pad_blocks == 0) {
+    if (t->mspad_blocks == 0) {
         return ISO_SUCCESS;
     }
 
     memset(pad, 0, BLOCK_SIZE);
-    for (i = 0; i < t->pad_blocks; ++i) {
+    for (i = 0; i < t->mspad_blocks; ++i) {
         ret = iso_write(t, pad, BLOCK_SIZE);
         if (ret < 0) {
             return ret;
@@ -948,14 +993,14 @@ int pad_writer_write_data(IsoImageWriter *writer)
 }
 
 static
-int pad_writer_free_data(IsoImageWriter *writer)
+int mspad_writer_free_data(IsoImageWriter *writer)
 {
     /* nothing to do */
     return ISO_SUCCESS;
 }
 
 static
-int pad_writer_create(Ecma119Image *target)
+int mspad_writer_create(Ecma119Image *target)
 {
     IsoImageWriter *writer;
 
@@ -964,10 +1009,10 @@ int pad_writer_create(Ecma119Image *target)
         return ISO_OUT_OF_MEM;
     }
 
-    writer->compute_data_blocks = pad_writer_compute_data_blocks;
-    writer->write_vol_desc = pad_writer_write_vol_desc;
-    writer->write_data = pad_writer_write_data;
-    writer->free_data = pad_writer_free_data;
+    writer->compute_data_blocks = mspad_writer_compute_data_blocks;
+    writer->write_vol_desc = mspad_writer_write_vol_desc;
+    writer->write_data = mspad_writer_write_data;
+    writer->free_data = mspad_writer_free_data;
     writer->data = NULL;
     writer->target = target;
 
@@ -976,6 +1021,100 @@ int pad_writer_create(Ecma119Image *target)
     return ISO_SUCCESS;
 }
 
+
+/** ----- Zero padding writer ----- */
+
+struct iso_zero_writer_data_struct {
+    uint32_t num_blocks;
+};
+
+static
+int zero_writer_compute_data_blocks(IsoImageWriter *writer)
+{
+    Ecma119Image *target;
+    struct iso_zero_writer_data_struct *data;
+
+    if (writer == NULL)
+        return ISO_ASSERT_FAILURE;
+    target = writer->target;
+    data = (struct iso_zero_writer_data_struct *) writer->data;
+    target->curblock += data->num_blocks;
+    return ISO_SUCCESS;
+}
+
+static
+int zero_writer_write_vol_desc(IsoImageWriter *writer)
+{
+    /* nothing to do */
+    return ISO_SUCCESS;
+}
+
+static
+int zero_writer_write_data(IsoImageWriter *writer)
+{
+    int ret;
+    Ecma119Image *t;
+    struct iso_zero_writer_data_struct *data;
+    uint8_t pad[BLOCK_SIZE];
+    size_t i;
+
+    if (writer == NULL)
+        return ISO_ASSERT_FAILURE;
+    t = writer->target;
+    data = (struct iso_zero_writer_data_struct *) writer->data;
+
+    if (data->num_blocks == 0) 
+        return ISO_SUCCESS;
+    memset(pad, 0, BLOCK_SIZE);
+    for (i = 0; i < data->num_blocks; ++i) {
+        ret = iso_write(t, pad, BLOCK_SIZE);
+        if (ret < 0) 
+            return ret;
+    }
+    return ISO_SUCCESS;
+}
+
+static
+int zero_writer_free_data(IsoImageWriter *writer)
+{
+    if (writer == NULL)
+        return ISO_SUCCESS;
+    if (writer->data == NULL)
+        return ISO_SUCCESS;
+    free(writer->data);
+    writer->data = NULL;
+    return ISO_SUCCESS;
+}
+
+static
+int zero_writer_create(Ecma119Image *target, uint32_t num_blocks)
+{
+    IsoImageWriter *writer;
+    struct iso_zero_writer_data_struct *data;
+
+    writer = malloc(sizeof(IsoImageWriter));
+    if (writer == NULL) {
+        return ISO_OUT_OF_MEM;
+    }
+    data = calloc(1, sizeof(struct iso_zero_writer_data_struct));
+    if (data == NULL) {
+        free(writer);
+        return ISO_OUT_OF_MEM;
+    }
+    data->num_blocks = num_blocks;
+
+    writer->compute_data_blocks = zero_writer_compute_data_blocks;
+    writer->write_vol_desc = zero_writer_write_vol_desc;
+    writer->write_data = zero_writer_write_data;
+    writer->free_data = zero_writer_free_data;
+    writer->data = data;
+    writer->target = target;
+
+    /* add this writer to image */
+    target->writers[target->nwriters++] = writer;
+    return ISO_SUCCESS;
+}
+
 static
 int transplant_checksum_buffer(Ecma119Image *target, int flag)
 {
@@ -1069,7 +1208,7 @@ int write_head_part2(Ecma119Image *target, int *write_count, int flag)
     if (target->partition_offset <= 0)
         return ISO_SUCCESS;
 
-    /* Write padding up to target->partition_offset + 16 */
+    /* Write multi-session padding up to target->partition_offset + 16 */
     memset(buf, 0, 2048);
     for(; *write_count < target->partition_offset + 16; (*write_count)++) {
         res = iso_write(target, buf, BLOCK_SIZE);
@@ -1136,6 +1275,62 @@ int write_head_part(Ecma119Image *target, int flag)
     return ISO_SUCCESS;
 }
 
+
+/* Eventually end Jigdo Template Extraction */
+static int finish_libjte(Ecma119Image *target)
+{
+#ifdef Libisofs_with_libjtE
+
+    int ret;
+
+    if (target->libjte_handle != NULL) {
+        ret = libjte_write_footer(target->libjte_handle);
+        if (ret <= 0) {
+            iso_libjte_forward_msgs(target->libjte_handle, 
+                                target->image->id, ISO_LIBJTE_END_FAILED, 0);
+            return ISO_LIBJTE_END_FAILED;
+        }
+    }
+
+#endif /* Libisofs_with_libjtE */
+
+    return 1;
+}
+
+
+static int write_mbr_partition_file(Ecma119Image *target, char *path,
+                                    uint32_t blocks, int flag)
+{
+    FILE *fp = NULL;
+    uint32_t i;
+    uint8_t buf[BLOCK_SIZE];
+    int ret;
+
+    fp = fopen(path, "rb");
+    if (fp == NULL)
+        return ISO_BAD_PARTITION_FILE;
+
+    for (i = 0; i < blocks; i++) {
+        memset(buf, 0, BLOCK_SIZE);
+        if (fp != NULL) {
+            ret = fread(buf, 1, BLOCK_SIZE, fp);
+            if (ret != BLOCK_SIZE) {
+                 fclose(fp);
+                 fp = NULL;
+            }
+        }
+        ret = iso_write(target, buf, BLOCK_SIZE);
+        if (ret < 0) {
+            fclose(fp);
+            return ret;
+        }
+    }
+    
+    fclose(fp);
+    return ISO_SUCCESS;
+}
+
+
 static
 void *write_function(void *arg)
 {
@@ -1162,11 +1357,32 @@ void *write_function(void *arg)
         }
     }
 
+    /* Append partition data */
+    for (i = 0; i < 4; i++) {
+        if (target->appended_partitions[i] == NULL)
+    continue;
+        res = write_mbr_partition_file(target, target->appended_partitions[i],
+                                       target->appended_part_size[i], 0);
+        if (res < 0)
+            goto write_error;
+    }
+
+
     /* Transplant checksum buffer from Ecma119Image to IsoImage */
     transplant_checksum_buffer(target, 0);
 
     iso_ring_buffer_writer_close(target->buffer, 0);
 
+    res = finish_libjte(target);
+    if (res <= 0)
+        goto write_error;
+
+    target->image->generator_is_running = 0;
+
+    /* Give up reference claim made in ecma119_image_new().
+       Eventually free target */
+    ecma119_image_free(target);
+
 #ifdef Libisofs_with_pthread_exiT
     pthread_exit(NULL);
 #else
@@ -1174,6 +1390,8 @@ void *write_function(void *arg)
 #endif
 
     write_error: ;
+    if (res != ISO_LIBJTE_END_FAILED)
+        finish_libjte(target);
     target->eff_partition_offset = 0;
     if (res == ISO_CANCELED) {
         /* canceled */
@@ -1190,6 +1408,12 @@ void *write_function(void *arg)
     /* Invalidate the transplanted checksum buffer in IsoImage */
     iso_image_free_checksums(target->image, 0);
 
+    target->image->generator_is_running = 0;
+
+    /* Give up reference claim made in ecma119_image_new().
+       Eventually free target */
+    ecma119_image_free(target);
+
 #ifdef Libisofs_with_pthread_exiT
     pthread_exit(NULL);
 #else
@@ -1292,7 +1516,6 @@ int checksum_prepare_nodes(Ecma119Image *target, IsoNode *node, int flag)
     return ISO_SUCCESS;
 }
 
-
 static
 int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
 {
@@ -1304,11 +1527,16 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     char *system_area = NULL;
     int write_count = 0, write_count_mem;
 
+
     /* 1. Allocate target and copy opts there */
     target = calloc(1, sizeof(Ecma119Image));
     if (target == NULL) {
         return ISO_OUT_OF_MEM;
     }
+    /* This reference will be transfered to the burn_source and released by
+       bs_free_data.
+    */
+    target->refcount = 1;
 
     /* create the tree for file caching */
     ret = iso_rbtree_new(iso_file_src_cmp, &(target->files));
@@ -1384,7 +1612,14 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     } else if (src->system_area_data != NULL) {
         system_area = src->system_area_data;
         system_area_options = src->system_area_options;
+    } else {
+        system_area_options = opts->system_area_options & 0xfc;
     }
+    if ((system_area_options & 0xfc) != 0)
+        for (i = 0; i < 4; i++)
+            if (opts->appended_partitions[i] != NULL)
+                return ISO_NON_MBR_SYS_AREA;
+
     target->system_area_data = NULL;
     if (system_area != NULL) {
         target->system_area_data = calloc(32768, 1);
@@ -1405,6 +1640,10 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target->partition_offset = opts->partition_offset;
     target->partition_secs_per_head = opts->partition_secs_per_head;
     target->partition_heads_per_cyl = opts->partition_heads_per_cyl;
+    if (target->partition_secs_per_head == 0)
+        target->partition_secs_per_head = 63;
+    if (target->partition_heads_per_cyl == 0)
+        target->partition_heads_per_cyl = 255;
     target->eff_partition_offset = 0;
     target->partition_root = NULL;
     target->partition_l_table_pos = 0;
@@ -1412,7 +1651,6 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target->j_part_root = NULL;
     target->j_part_l_path_table_pos = 0;
     target->j_part_m_path_table_pos = 0;
-
     target->input_charset = strdup(iso_get_local_charset(0));
     if (target->input_charset == NULL) {
         ret = ISO_OUT_OF_MEM;
@@ -1446,6 +1684,42 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target->checksum_range_size = 0;
     target->opts_overwrite = NULL;
 
+#ifdef Libisofs_with_libjtE
+
+    /* Eventually start Jigdo Template Extraction */
+    target->libjte_handle = NULL;
+    if (opts->libjte_handle != NULL) {
+        target->libjte_handle = opts->libjte_handle;
+        ret = libjte_write_header(target->libjte_handle);
+        if (ret <= 0) {
+            iso_libjte_forward_msgs(target->libjte_handle, 
+                                target->image->id, ISO_LIBJTE_START_FAILED, 0);
+            ret = ISO_LIBJTE_START_FAILED;
+            goto target_cleanup;
+        }
+    }
+    
+#endif /* Libisofs_with_libjtE */
+
+    target->tail_blocks = opts->tail_blocks;
+
+    target->mipsel_e_entry = 0;
+    target->mipsel_p_offset = 0;
+    target->mipsel_p_vaddr = 0;
+    target->mipsel_p_filesz = 0;
+
+    for (i = 0; i < 4; i++) {
+        if (opts->appended_partitions[i] != NULL) {
+            target->appended_partitions[i] =
+                                          strdup(opts->appended_partitions[i]);
+            if (target->appended_partitions[i] == NULL)
+                return ISO_OUT_OF_MEM;
+            target->appended_part_types[i] = opts->appended_part_types[i];
+        }
+        target->appended_part_start[i] = target->appended_part_size[i] = 0;
+    }
+
+
     /*
      * 2. Based on those options, create needed writers: iso, joliet...
      * Each writer inits its structures and stores needed info into
@@ -1464,7 +1738,7 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     }
 
     /* the number of writers is dependent of the extensions */
-    nwriters = 1 + 1 + 1; /* ECMA-119 + padding + files */
+    nwriters = 1 + 1 + 1; /* ECMA-119 + multi-session padding + files */
 
     if (target->eltorito) {
         nwriters++;
@@ -1475,7 +1749,8 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     if (target->iso1999) {
         nwriters++;
     }
-
+    if (target->tail_blocks > 0) 
+        nwriters++;
     if ((target->md5_file_checksums & 1) || target->md5_session_checksum) {
         nwriters++;
         image_checksums_mad = 1; /* from here on the loaded checksums are
@@ -1538,7 +1813,7 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
      * Create the writer for possible padding to ensure that in case of image
      * growing we can safely overwrite the first 64 KiB of image.
      */
-    ret = pad_writer_create(target);
+    ret = mspad_writer_create(target);
     if (ret < 0) {
         goto target_cleanup;
     }
@@ -1549,6 +1824,15 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
         goto target_cleanup;
     }
     file_src_writer_index = target->nwriters - 1;
+
+
+    /* IMPORTANT: This must be the last writer before the checksum writer */
+    if (target->tail_blocks > 0) {
+        ret = zero_writer_create(target, target->tail_blocks);
+        if (ret < 0)
+            goto target_cleanup;
+    }
+
     if ((target->md5_file_checksums & 1) || target->md5_session_checksum) {
         ret = checksum_writer_create(target);
         if (ret < 0)
@@ -1597,12 +1881,9 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     for (i = 0; i < target->nwriters; ++i) {
         IsoImageWriter *writer = target->writers[i];
 
-#define Libisofs_patch_ticket_145 yes
-#ifdef Libisofs_patch_ticket_145
         /* Delaying boot image patching until new LBA is known */
         if (i == el_torito_writer_index)
     continue;
-#endif
 
         /* Exposing address of data start to IsoWriteOpts */
         if (i == file_src_writer_index) {
@@ -1614,8 +1895,8 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
             goto target_cleanup;
         }
     }
-#ifdef Libisofs_patch_ticket_145
-    /* Now perform delayed image patching */
+
+    /* Now perform delayed image patching and System Area preparations */
     if (el_torito_writer_index >= 0) {
         IsoImageWriter *writer = target->writers[el_torito_writer_index];
         ret = writer->compute_data_blocks(writer);
@@ -1623,7 +1904,23 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
             goto target_cleanup;
         }
     }
-#endif /* Libisofs_patch_ticket_145 */
+    if (((target->system_area_options & 0xfc) >> 2) == 2) {
+        ret = iso_read_mipsel_elf(target, 0);
+        if (ret < 0)
+            goto target_cleanup;
+    }
+
+    /*
+     * The volume space size is just the size of the last session, in
+     * case of ms images.
+     */
+    target->vol_space_size = target->curblock - target->ms_block;
+    target->total_size = (off_t) target->vol_space_size * BLOCK_SIZE;
+
+    /* Add sizes of eventually appended partitions */
+    ret = iso_compute_append_partitions(target, 0);
+    if (ret < 0)
+        goto target_cleanup;
 
     /* create the ring buffer */
     if (opts->overwrite != NULL &&
@@ -1718,13 +2015,8 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
         }
     }
 
-    /*
-     * The volume space size is just the size of the last session, in
-     * case of ms images.
-     */
+    /* This was possibly altered by above overwrite buffer production */
     target->vol_space_size = target->curblock - target->ms_block;
-    target->total_size = (off_t) target->vol_space_size * BLOCK_SIZE;
-
 
     /* 4. Create and start writing thread */
     if (target->md5_session_checksum) {
@@ -1744,9 +2036,24 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     pthread_attr_init(&(target->th_attr));
     pthread_attr_setdetachstate(&(target->th_attr), PTHREAD_CREATE_JOINABLE);
 
+    /* To avoid race conditions with the caller, this mark must be set
+       before the thread starts. So the caller can rely on a value of 0
+       really meaning that the write has ended, and not that it might not have
+       begun yet.
+       In normal processing, the value will be changed to 0 at the end of
+       write_function().
+    */
+    target->image->generator_is_running = 1;
+
+
+    /* Claim that target may not get destroyed by bs_free_data() before
+       write_function() is done with it */
+    target->refcount++;
+
     ret = pthread_create(&(target->wthread), &(target->th_attr),
                          write_function, (void *) target);
     if (ret != 0) {
+        target->refcount--;
         iso_msg_submit(target->image->id, ISO_THREAD_ERROR, 0,
                       "Cannot create writer thread");
         ret = ISO_THREAD_ERROR;
@@ -1766,6 +2073,7 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target_cleanup: ;
     if(image_checksums_mad) /* No checksums is better than mad checksums */
       iso_image_free_checksums(target->image, 0);
+    target->image->generator_is_running = 0;
     ecma119_image_free(target);
     return ret;
 }
@@ -1816,7 +2124,7 @@ static void bs_free_data(struct burn_source *bs)
                   iso_ring_buffer_get_times_full(target->buffer),
                   iso_ring_buffer_get_times_empty(target->buffer));
 
-    /* now we can safety free target */
+    /* The reference to target was inherited from ecma119_image_new() */
     ecma119_image_free(target);
 }
 
@@ -1910,6 +2218,11 @@ int iso_write(Ecma119Image *target, void *buf, size_t count)
         target->checksum_counter += count;
         iso_md5_compute(target->checksum_ctx, (char *) buf, (int) count);
     }
+
+    ret = show_chunk_to_jte(target, buf, count);
+    if (ret != ISO_SUCCESS)
+        return ret;
+
     /* total size is 0 when writing the overwrite buffer */
     if (ret > 0 && (target->total_size != (off_t) 0)){
         unsigned int kbw, kbt;
@@ -1933,6 +2246,7 @@ int iso_write(Ecma119Image *target, void *buf, size_t count)
 
 int iso_write_opts_new(IsoWriteOpts **opts, int profile)
 {
+    int i;
     IsoWriteOpts *wopts;
 
     if (opts == NULL) {
@@ -1987,19 +2301,32 @@ int iso_write_opts_new(IsoWriteOpts **opts, int profile)
     wopts->partition_secs_per_head = 0;
     wopts->partition_heads_per_cyl = 0;
 
+#ifdef Libisofs_with_libjtE
+    wopts->libjte_handle = NULL;
+#endif /* Libisofs_with_libjtE */
+
+    wopts->tail_blocks = 0;
+    for (i = 0; i < 4; i++)
+        wopts->appended_partitions[i] = NULL;
+
     *opts = wopts;
     return ISO_SUCCESS;
 }
 
 void iso_write_opts_free(IsoWriteOpts *opts)
 {
+    int i;
+
     if (opts == NULL) {
         return;
     }
-
     free(opts->output_charset);
-    if(opts->system_area_data != NULL)
+    if (opts->system_area_data != NULL)
         free(opts->system_area_data);
+    for (i = 0; i < 4; i++)
+        if (opts->appended_partitions[i] != NULL)
+            free(opts->appended_partitions[i]);
+
     free(opts);
 }
 
@@ -2375,9 +2702,24 @@ int iso_write_opts_get_data_start(IsoWriteOpts *opts, uint32_t *data_start,
 
 /*
  * @param data     Either NULL or 32 kB of data. Do not submit less bytes !
- * @param options  bit0 = apply GRUB protective msdos label
+ * @param options 
+ *        Can cause manipulations of submitted data before they get written:
+ *        bit0= apply a --protective-msdos-label as of grub-mkisofs.
+ *              This means to patch bytes 446 to 512 of the system area so
+ *              that one partition is defined which begins at the second
+ *              512-byte block of the image and ends where the image ends.
+ *              This works with and without system_area_data.
+ *        bit1= apply isohybrid MBR patching to the system area.
+ *              This works only with system area data from SYSLINUX plus an
+ *              ISOLINUX boot image (see iso_image_set_boot_image()) and
+ *              only if not bit0 is set.
+ *        bit2-7= System area type
+ *              0= PC-BIOS DOS MBR
+ *              1= MIPS Big Endian Volume Header
  * @param flag     bit0 = invalidate any attached system area data
  *                        same as data == NULL
+ *                 bit1 = keep data unaltered
+ *                 bit2 = keep options unaltered
  */
 int iso_write_opts_set_system_area(IsoWriteOpts *opts, char data[32768],
                                    int options, int flag)
@@ -2395,7 +2737,7 @@ int iso_write_opts_set_system_area(IsoWriteOpts *opts, char data[32768],
         memcpy(opts->system_area_data, data, 32768);
     }
     if (!(flag & 4))
-        opts->system_area_options = options & 3;
+        opts->system_area_options = options & 0xff;
     return ISO_SUCCESS;
 }
 
@@ -2424,4 +2766,56 @@ int iso_write_opts_set_part_offset(IsoWriteOpts *opts,
     opts->partition_heads_per_cyl = heads_per_cyl;
     return ISO_SUCCESS;
 }
-                                  
+
+int iso_write_opts_attach_jte(IsoWriteOpts *opts, void *libjte_handle)
+{
+#ifdef Libisofs_with_libjtE
+
+    opts->libjte_handle = libjte_handle;
+    return ISO_SUCCESS;
+
+#else
+
+    return ISO_LIBJTE_NOT_ENABLED;
+
+#endif /* Libisofs_with_libjtE */
+}
+
+int iso_write_opts_detach_jte(IsoWriteOpts *opts, void **libjte_handle)
+{
+#ifdef Libisofs_with_libjtE
+
+    if (*libjte_handle != NULL)
+        *libjte_handle = opts->libjte_handle;
+    opts->libjte_handle = NULL;
+    return ISO_SUCCESS;
+
+#else
+
+    return ISO_LIBJTE_NOT_ENABLED;
+
+#endif /* Libisofs_with_libjtE */
+}
+ 
+int iso_write_opts_set_tail_blocks(IsoWriteOpts *opts, uint32_t num_blocks)
+{
+    opts->tail_blocks = num_blocks;
+    return ISO_SUCCESS;
+}
+
+int iso_write_opts_set_partition_img(IsoWriteOpts *opts, int partition_number,
+                            uint8_t partition_type, char *image_path, int flag)
+{
+    if (partition_number < 1 || partition_number > 4)
+        return ISO_BAD_PARTITION_NO;
+
+    if (opts->appended_partitions[partition_number - 1] != NULL)
+        free(opts->appended_partitions[partition_number - 1]);
+    opts->appended_partitions[partition_number - 1] = strdup(image_path);
+    if (opts->appended_partitions[partition_number - 1] == NULL)
+        return ISO_OUT_OF_MEM;
+    opts->appended_part_types[partition_number - 1] = partition_type;
+
+    return ISO_SUCCESS;
+}
+
