@@ -324,6 +324,21 @@ struct iso_write_opts {
     /* 1 to 255, 0= disabled/default */
     int partition_heads_per_cyl;
 
+#ifdef Libisofs_with_libjtE
+    /* Parameters and state of Jigdo Template Export environment.
+    */
+    struct libjte_env *libjte_handle;
+#endif /* Libisofs_with_libjtE */
+
+    /* A trailing padding of zero bytes which belongs to the image
+    */
+    uint32_t tail_blocks;
+
+    /* Eventual disk file paths of prepared images which shall be appended
+       after the ISO image and described by partiton table entries in a MBR
+    */
+    char *appended_partitions[4]; 
+    uint8_t appended_part_types[4];
 };
 
 typedef struct ecma119_image Ecma119Image;
@@ -335,6 +350,8 @@ typedef struct Iso_Image_Writer IsoImageWriter;
 
 struct ecma119_image
 {
+    int refcount;
+
     IsoImage *image;
     Ecma119Node *root;
 
@@ -469,14 +486,19 @@ struct ecma119_image
      */
     char *system_area_data;
     /*
-     * bit0= make bytes 446 - 512 of the system area a partition
+     * bit0= Only with PC-BIOS DOS MBR
+     *       Make bytes 446 - 512 of the system area a partition
      *       table which reserves partition 1 from byte 63*512 to the
      *       end of the ISO image. Assumed are 63 secs/hed, 255 head/cyl.
      *       (GRUB protective msdos label.)
      *       This works with and without system_area_data.
-     * bit1= apply isohybrid MBR patching to the system area.
+     * bit1= Only with PC-BIOS DOS MBR
+     *       Apply isohybrid MBR patching to the system area.
      *       This works only with system_area_data plus ISOLINUX boot image
      *       and only if not bit0 is set.
+     * bit2-7= System area type
+     *       0= DOS MBR
+     *       1= MIPS Big Endian Volume Header
      */
     int system_area_options;
 
@@ -493,7 +515,7 @@ struct ecma119_image
      * file data is written in the first 64 KiB, that are the bytes we usually
      * overwrite.
      */
-    uint32_t pad_blocks;
+    uint32_t mspad_blocks;
 
     size_t nwriters;
     IsoImageWriter **writers;
@@ -567,6 +589,24 @@ struct ecma119_image
     uint32_t j_part_l_path_table_pos;
     uint32_t j_part_m_path_table_pos;
 
+#ifdef Libisofs_with_libjtE
+    struct libjte_env *libjte_handle;
+#endif /* Libisofs_with_libjtE */
+
+    uint32_t tail_blocks;
+
+    /* Memorized ELF parameters from MIPS Little Endian boot file */
+    uint32_t mipsel_e_entry;
+    uint32_t mipsel_p_offset;
+    uint32_t mipsel_p_vaddr;
+    uint32_t mipsel_p_filesz;
+
+    char *appended_partitions[4]; 
+    uint8_t appended_part_types[4];
+    /* Counted in blocks of 2048 */
+    uint32_t appended_part_start[4];
+    uint32_t appended_part_size[4];
+
 };
 
 #define BP(a,b) [(b) - (a) + 1]
