@@ -1,3 +1,8 @@
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
 #include <ctype.h>
 #include <stdint.h>
 #include <sys/types.h>
