@@ -25,6 +25,21 @@
 #include <sys/time.h>
 #include <time.h>
 #include <errno.h>
+#include <stdint.h>
+
+#ifdef Xorriso_standalonE
+
+#ifdef Xorriso_with_libjtE
+#include "../libjte/libjte.h"
+#endif
+
+#else
+
+#ifdef Xorriso_with_libjtE
+#include <libjte/libjte.h>
+#endif
+
+#endif /* ! Xorriso_standalonE */
 
 #include "xorriso.h"
 #include "xorriso_private.h"
@@ -210,11 +225,22 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->partition_offset= 0;
  m->partition_secs_per_head= 0;
  m->partition_heads_per_cyl= 0;
+ for(i= 0; i < 4; i++) {
+   m->appended_partitions[i][0]= 0;
+   m->appended_part_types[i]= 0;
+ }
  m->vol_creation_time= 0;
  m->vol_modification_time= 0;
  m->vol_expiration_time= 0;
  m->vol_effective_time= 0;
  m->vol_uuid[0]= 0;
+
+#ifdef Xorriso_with_libjtE
+ m->libjte_handle= NULL;
+#endif
+
+ m->jigdo_params= NULL;
+ m->jigdo_values= NULL;
  m->loaded_boot_bin_lba= 0;
  m->loaded_boot_cat_path[0]= 0;
  m->allow_graft_points= 0;
@@ -417,6 +443,13 @@ int Xorriso_destroy(struct XorrisO **xorriso, int flag)
  Xorriso_destroy_hln_array(m, 0);
  Xorriso_destroy_di_array(m, 0);
 
+#ifdef Xorriso_with_libjtE
+ if(m->libjte_handle)
+   libjte_destroy(&(m->libjte_handle));
+#endif
+
+ Xorriso_lst_destroy_all(&(m->jigdo_params), 0);
+ Xorriso_lst_destroy_all(&(m->jigdo_values), 0);
  Xorriso_detach_libraries(m, flag&1);
 
  free((char *) m);
