@@ -1157,7 +1157,7 @@ int Xorriso_tell_media_space(struct XorrisO *xorriso,
 */
 int Xorriso_list_formats(struct XorrisO *xorriso, int flag)
 {
- int ret, i, status, num_formats, profile_no, type;
+ int ret, i, status, num_formats, profile_no, type, alloc_blocks, free_blocks;
  off_t size;
  unsigned dummy;
  char status_text[80], profile_name[90], *respt;
@@ -1206,6 +1206,12 @@ int Xorriso_list_formats(struct XorrisO *xorriso, int flag)
    sprintf(status_text, "illegal status according to MMC-5");
  sprintf(respt, "Format status: %s\n", status_text);
  Xorriso_result(xorriso,0);
+ ret= burn_disc_get_bd_spare_info(drive, &alloc_blocks, &free_blocks, 0);
+ if(ret == 1) {
+   sprintf(respt, "BD Spare Area: %d blocks consumed, %d blocks available\n",
+           alloc_blocks - free_blocks, free_blocks);
+   Xorriso_result(xorriso,0);
+ }
 
  for (i= 0; i < num_formats; i++) {
    ret= burn_disc_get_format_descr(drive, i, &type, &size, &dummy);
@@ -2003,8 +2009,8 @@ int Xorriso_check_media(struct XorrisO *xorriso, struct SpotlisT **spotlist,
  int media_blocks= 0, read_chunk= 16, ret, mode, start_lba= 0;
  int blocks, os_errno, i, j, last_track_end= -1, track_blocks, track_lba;
  int num_sessions, num_tracks, declare_untested= 0, md5_start;
- int read_capacity= -1, end_lba, hret, count, quality;
- char *toc_info= NULL;
+ int read_capacity= -1, end_lba, hret, count, quality, profile_no;
+ char *toc_info= NULL, profile_name[80];
  struct burn_drive *drive;
  struct burn_drive_info *dinfo;
  struct isoburn_toc_disc *isoburn_disc= NULL;
@@ -2063,6 +2069,9 @@ int Xorriso_check_media(struct XorrisO *xorriso, struct SpotlisT **spotlist,
  ret= burn_get_read_capacity(drive, &read_capacity, 0);
  if(ret <= 0)
    read_capacity= -1;
+ ret = burn_disc_get_profile(drive, &profile_no, profile_name);
+ if(ret <= 0)
+   profile_no= 0;
 
  if(job->max_lba >= 0) {
    blocks= job->max_lba + 1 - start_lba;
@@ -2087,6 +2096,16 @@ int Xorriso_check_media(struct XorrisO *xorriso, struct SpotlisT **spotlist,
      continue;
        track_lba= isoburn_entry.start_lba;
        track_blocks= isoburn_entry.track_blocks;
+
+       /* The last track of an appendable BD-R reports more blocks than the
+          read capacity allows. All BD-R track sizes are multiple of 64 kB.
+       */
+       if (i == num_sessions - 1 && 
+           (track_lba + track_blocks > read_capacity &&
+            track_lba + track_blocks < read_capacity + 32 &&
+            (profile_no == 0x41 || profile_no == 0x40)))
+         track_blocks= read_capacity - track_lba;
+
        md5_start= track_lba;
        if(i == 0 && j == 0) {
          if(track_lba == 32) {
