@@ -559,6 +559,8 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "-hppa-bootloader", "-hppa-ramdisk", "-mips-boot", "-mipsel-boot",
     "-jigdo-jigdo", "-jigdo-template", "-jigdo-min-file-size",
     "-jigdo-force-md5", "-jigdo-exclude", "-jigdo-map", "-md5-list",
+    "-jigdo-template-compress",
+    "-checksum_algorithm_iso", "-checksum_algorithm_template",
     "--stdio_sync", "--quoted_path_list", "--efi-boot", "--embedded-boot",
     "-isohybrid-mbr", "-e", "-partition_offset", "-partition_hd_cyl",
     "-partition_sec_hd",
@@ -568,6 +570,10 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "--scdbackup_tag", "--sort-weight",
     ""
  };
+ static char arg3_options[][41]= {
+    "-append_partition",
+    ""
+ };
  static char final_options[][41]= {
     "-find",
     ""
@@ -589,6 +595,10 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
  for(i=0; arg2_options[i][0]!=0; i++)
    if(strcmp(arg2_options[i], cmd)==0)
      return(1);
+ *count= 3;
+ for(i=0; arg3_options[i][0]!=0; i++)
+   if(strcmp(arg3_options[i], cmd)==0)
+     return(1);
  *count= argc - 1;
  for(i=0; final_options[i][0]!=0; i++)
    if(strcmp(final_options[i], cmd)==0)
@@ -712,6 +722,20 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -biblio FILE                Set Bibliographic filename",
 "  -copyright FILE             Set Copyright filename",
 "  -p PREP, -preparer PREP     Set Volume preparer",
+"  -jigdo-jigdo FILE           Produce a jigdo .jigdo file as well as the .iso",
+"  -jigdo-template FILE        Produce a jigdo .template file as well as the .iso",
+"  -jigdo-min-file-size SIZE   Minimum size for a file to be listed in the jigdo file", 
+"  -jigdo-force-md5 PATTERN    Pattern(s) where files MUST match an externally-supplied MD5sum",
+"  -jigdo-exclude PATTERN      Pattern(s) to exclude from the jigdo file",
+"  -jigdo-map PATTERN1=PATTERN2",
+"                              Pattern(s) to map paths (e.g. Debian=/mirror/debian)",
+"  -md5-list FILE              File containing MD5 sums of the files that should be checked",
+"  -jigdo-template-compress ALGORITHM",
+"                              Choose to use gzip or bzip2 compression for template data; default is gzip",
+"  -checksum_algorithm_iso alg1,alg2,...",
+"                              Specify the checksum types desired for the output image (in .jigdo)",
+"  -checksum_algorithm_template alg1,alg2,...",
+"                              Specify the checksum types desired for the output jigdo template"
 "  -b FILE, -eltorito-boot FILE",
 "                              Set El Torito boot image name",
 "  -eltorito-alt-boot          Start specifying alternative El Torito boot parameters",
@@ -729,6 +753,9 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -partition_offset LBA       Make image mountable by first partition, too",
 "  -partition_sec_hd NUMBER    Define number of sectors per head",
 "  -partition_hd_cyl NUMBER    Define number of heads per cylinder",
+"  -mips-boot FILE             Set mips boot image name (relative to image root)",
+"  -append_partition NUMBER TYPE FILE",
+"                              Append FILE after image. TYPE is hex: 0x..",
 "  --modification-date=YYYYMMDDhhmmsscc",
 "                              Override date of creation and modification",
 "  -isohybrid-mbr FILE         Set SYSLINUX mbr/isohdp[fp]x*.bin for isohybrid",
@@ -1223,6 +1250,8 @@ not_enough_args:;
              strcmp(argv[i], "-eltorito-boot") == 0 ||
              strcmp(argv[i], "--efi-boot") == 0 ||
              strcmp(argv[i], "-e") == 0 ||
+             strcmp(argv[i], "-mips-boot") == 0 ||
+             strcmp(argv[i], "-mipsel-boot") == 0 ||
              strcmp(argv[i], "-c") == 0 ||
              strcmp(argv[i], "-eltorito-catalog") == 0 ||
              strcmp(argv[i], "-boot-load-size") == 0 ||
@@ -1310,6 +1339,27 @@ not_enough_args:;
      if(ret<=0)
        goto problem_handler_2;
      file_mode= mode_or;
+   } else if(strcmp(argv[i], "-jigdo-jigdo") == 0 ||
+             strcmp(argv[i], "-jigdo-template") == 0 ||     
+             strcmp(argv[i], "-jigdo-min-file-size") == 0 ||     
+             strcmp(argv[i], "-jigdo-exclude") == 0 ||     
+             strcmp(argv[i], "-jigdo-force-md5") == 0 ||     
+             strcmp(argv[i], "-jigdo-map") == 0 ||     
+             strcmp(argv[i], "-jigdo-template-compress") == 0 ||     
+             strcmp(argv[i], "-checksum_algorithm_iso") == 0 ||     
+             strcmp(argv[i], "-checksum_algorithm_template") == 0 ||     
+             strcmp(argv[i], "-md5-list") == 0) {
+     i++;
+     ret= Xorriso_option_jigdo(xorriso, argv[i - 1], argv[i], 0);
+     if(ret <= 0)
+       goto problem_handler_2;
+   } else if(strcmp(argv[i], "-append_partition") == 0) {
+     i+= 3;
+     ret= Xorriso_option_append_partition(xorriso, argv[i - 2], argv[i - 1],
+                                          argv[i], 0);
+     if(ret <= 0)
+       goto problem_handler_2;
+
    } else if(argv[i][0]=='-' && argv[i][1]!=0) {
      ret= Xorriso_genisofs_count_args(xorriso, argc - i, argv + i, &count, 1);
      if(ret > 0) {
@@ -1436,6 +1486,7 @@ problem_handler_2:;
        boot_path= xorriso->boot_image_bin_path;
        xorriso->boot_efi_default= 1;
        emul_boot= xorriso->boot_image_emul= 0;
+
      } else {
        boot_path= xorriso->boot_image_bin_path; 
        if(strcmp(argv[i - 1], "-e") == 0)
@@ -1518,6 +1569,24 @@ problem_handler_2:;
      ret= Xorriso_option_boot_image(xorriso, "any", sfe, 0);
      if(ret <= 0)
        goto problem_handler_boot;
+
+   } else if(strcmp(argv[i], "-mips-boot") == 0 ||
+             strcmp(argv[i], "-mipsel-boot") == 0) {
+     if(i + 1 >= argc)
+       goto not_enough_args;
+     i++;
+     if(strcmp(argv[i - 1], "-mipsel-boot") == 0)
+       strcpy(sfe, "mipsel_path=");
+     else
+       strcpy(sfe, "mips_path=");
+     ret= Sfile_str(sfe, argv[i], 1);
+     if(ret <= 0)
+       goto ex;
+
+     ret = Xorriso_option_boot_image(xorriso, "any", sfe, 0);
+     if(ret <= 0)
+       goto problem_handler_boot;
+
    }
  continue; /* regular bottom of loop */
 problem_handler_boot:;
