@@ -752,7 +752,7 @@ improper_range:;
 }
 
 
-/* @param flag bit0= -wholename rather than -name
+/* @param flag bit0-1=  0= -name , 1= -wholename , 2= -disk_name
 */
 int Findjob_set_name_expr(struct FindjoB *o, char *name_expr, int flag)
 {
@@ -768,7 +768,11 @@ int Findjob_set_name_expr(struct FindjoB *o, char *name_expr, int flag)
  if(ret <= 0)
    return(ret);
  t= o->cursor->test;
- t->test_type= (flag & 1 ? 13 : 1);
+ t->test_type= 1;
+ if ((flag & 3) == 1)
+   t->test_type= 13;
+ else if((flag & 3) == 2)
+   t->test_type= 16;
  name_re= (regex_t *) calloc(1, sizeof(regex_t));
  if(name_re == NULL)
    return(-1);
@@ -866,6 +870,25 @@ int Findjob_set_lba_range(struct FindjoB *o, int start_lba, int count,
 }
 
 
+int Findjob_set_test_hidden(struct FindjoB *o, int mode, int flag)
+{
+ struct ExprtesT *t;
+ int ret;
+
+ ret= Findjob_default_and(o, 0);
+ if(ret <= 0)
+   return(ret);
+
+ t= o->cursor->test;
+ t->test_type= 17;
+ t->arg1= calloc(sizeof(int), 1);
+ if(t->arg1 == NULL)
+   return(-1);
+ *((int *) t->arg1)= mode;
+ return(1);
+}
+
+
 /* @param value -1= files without ACL, 1= only files with ACL
 */
 int Findjob_set_acl_filter(struct FindjoB *o, int value, int flag)
