@@ -304,8 +304,10 @@ cannot_open_dir:;
  name= disk_path+strlen(disk_path);
  if(Sfile_str(img_path, img_dir_path, 0)<=0)
    {ret= -1; goto ex;}
- if(img_path[0] || img_path[strlen(img_path)-1]!='/')
-   strcat(img_path,"/");
+ if(img_path[0] == 0)
+   strcat(img_path, "/");
+ else if(img_path[strlen(img_path) - 1] != '/')
+   strcat(img_path, "/");
  img_name= img_path+strlen(img_path);
 
  while(1) { /* loop over directory content */
