@@ -23,11 +23,25 @@
 #include <sys/time.h>
 #include <time.h>
 #include <errno.h>
+#include <stdint.h>
 
 /* for -charset */
 #include <iconv.h>
 #include <langinfo.h>
 
+#ifdef Xorriso_standalonE
+
+#ifdef Xorriso_with_libjtE
+#include "../libjte/libjte.h"
+#endif
+
+#else
+
+#ifdef Xorriso_with_libjtE
+#include <libjte/libjte.h>
+#endif
+
+#endif /* ! Xorriso_standalonE */
 
 #include "xorriso.h" 
 #include "xorriso_private.h"
@@ -275,6 +289,16 @@ int Xorriso_report_lib_versions(struct XorrisO *xorriso, int flag)
          "libisofs   in use :  %d.%d.%d  (min. %d.%d.%d)\n",
          major, minor, micro, req_major, req_minor, req_micro);
  Xorriso_result(xorriso, 0);
+
+#ifdef Xorriso_with_libjtE
+ libjte__version(&major, &minor, &micro);
+ isoburn_libjte_req(&req_major, &req_minor, &req_micro);
+ sprintf(xorriso->result_line,
+         "libjte     in use :  %d.%d.%d  (min. %d.%d.%d)\n",
+         major, minor, micro, req_major, req_minor, req_micro);
+ Xorriso_result(xorriso, 0);
+#endif
+
  burn_version(&major, &minor, &micro);
  isoburn_libburn_req(&req_major, &req_minor, &req_micro);
  sprintf(xorriso->result_line,
@@ -413,14 +437,36 @@ cannot:;
 int Xorriso_process_msg_queues(struct XorrisO *xorriso, int flag)
 {
  int ret, error_code= 0, os_errno= 0, count= 0, pass, imgid, tunneled;
+ int name_prefix_code;
  char severity[80];
 
+#ifdef Xorriso_with_libjtE
+ char *msg;
+#endif
+
  if(!xorriso->libs_are_started)
    return(1);
- for(pass= 0; pass< 2; pass++) {
+ for(pass= 0; pass< 3; pass++) {
    while(1) {
      tunneled= 0;
-     if(pass==0)
+     if(pass==0) {
+
+       ret= 0;
+#ifdef Xorriso_with_libjtE
+       if(xorriso->libjte_handle != NULL) {
+         msg= libjte_get_next_message(xorriso->libjte_handle);
+         if(msg != NULL) {
+           sprintf(xorriso->info_text, "%1.4095s", msg);
+           free(msg);
+           strcpy(severity, "NOTE");
+           error_code= 0;
+           os_errno= 0;
+           ret= 1;
+         }
+       }
+#endif /* Xorriso_with_libjtE */
+
+     } else if(pass==1)
        ret= iso_obtain_msgs("ALL", &error_code, &imgid,
                             xorriso->info_text, severity);
      else {
@@ -444,8 +490,12 @@ int Xorriso_process_msg_queues(struct XorrisO *xorriso, int flag)
      else if(error_code==0x51001)
        strcpy(severity, "ERRFILE");
 
+     if(pass == 0)
+       name_prefix_code= 0;
+     else
+       name_prefix_code= pass + tunneled;
      Xorriso_msgs_submit(xorriso, error_code, xorriso->info_text, os_errno, 
-                         severity, ((pass+tunneled)+1)<<2);
+                         severity, name_prefix_code << 2);
      count++;
    }
  }
@@ -499,7 +549,7 @@ int Xorriso_preparer_string(struct XorrisO *xorriso, char xorriso_id[129],
  int major, minor, micro;
 
  xorriso_id[0]= 0;
- sprintf(xorriso_id, "XORRISO-%d.%d.%d",
+ sprintf(xorriso_id, "XORRISO-%d.%d.%d ",
          Xorriso_header_version_majoR, Xorriso_header_version_minoR,
          Xorriso_header_version_micrO);
  if(strlen(xorriso_id) + strlen(Xorriso_timestamP) < 128)
@@ -521,3 +571,159 @@ int Xorriso_preparer_string(struct XorrisO *xorriso, char xorriso_id[129],
  return(1);
 }
 
+
+#ifdef Xorriso_with_libjtE
+
+int Xorriso_assert_jte_handle(struct XorrisO *xorriso, int flag)
+{
+ int ret;
+ 
+ if(xorriso->libjte_handle == NULL) {
+   ret= libjte_new(&(xorriso->libjte_handle), 0);
+   if(ret <= 0 || xorriso->libjte_handle == NULL) {
+     sprintf(xorriso->info_text,
+             "-jigdo: Failed to create libjte environment object"); 
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FATAL", 0);
+     return(-1);
+   }
+   /* no stderr, no exit() */
+   libjte_set_error_behavior(xorriso->libjte_handle, 0, 0);
+ }
+ return(1);
+}
+
+#endif /* Xorriso_with_libjtE */
+
+
+int Xorriso_jigdo_interpreter(struct XorrisO *xorriso, char *aspect, char *arg,
+                         int flag)
+{
+
+#ifdef Xorriso_with_libjtE
+
+ int ret, num;
+ struct libjte_env *jte;
+ char *msg = NULL;
+
+ if(strcmp(aspect, "clear") == 0) {
+   if(xorriso->libjte_handle != NULL)
+   libjte_destroy(&(xorriso->libjte_handle));
+   Xorriso_lst_destroy_all(&(xorriso->jigdo_params), 0);
+   Xorriso_lst_destroy_all(&(xorriso->jigdo_values), 0);
+   return(1);
+ }
+ ret= Xorriso_assert_jte_handle(xorriso, 0);
+ if(ret <= 0)
+   return(ret);
+ jte= xorriso->libjte_handle;
+
+ if(strcmp(aspect, "verbose") == 0) {
+   if(strcmp(arg, "on") == 0) {
+     libjte_set_verbose(jte, 1);
+     /* Direct libjte messages to stderr, rather than message list */
+     libjte_set_error_behavior(xorriso->libjte_handle, 1, 0);
+   } else if(strcmp(arg, "off") == 0) {
+     libjte_set_verbose(jte, 0);
+     libjte_set_error_behavior(xorriso->libjte_handle, 0, 0);
+   } else
+     goto bad_arg;
+ } else if(strcmp(aspect, "template_path") == 0 ||
+           strcmp(aspect, "-jigdo-template") == 0) {
+   ret= libjte_set_template_path(jte, arg);
+   if(ret <= 0)
+     goto jte_failed;
+ } else if(strcmp(aspect, "jigdo_path") == 0 ||
+           strcmp(aspect, "-jigdo-jigdo") == 0) {
+   ret= libjte_set_jigdo_path(jte, arg);
+   if(ret <= 0)
+     goto jte_failed;
+ } else if(strcmp(aspect, "md5_path") == 0 ||
+           strcmp(aspect, "-md5-list") == 0) {
+   ret= libjte_set_md5_path(jte, arg);
+   if(ret <= 0)
+     goto jte_failed;
+ } else if(strcmp(aspect, "min_size") == 0 ||
+           strcmp(aspect, "-jigdo-min-file-size") == 0) {
+   num= Scanf_io_size(arg, 0);
+   ret= libjte_set_min_size(jte, num);
+   if(ret <= 0)
+     goto jte_failed;
+ } else if(strcmp(aspect, "checksum_iso") == 0 ||
+           strcmp(aspect, "-checksum_algorithm_iso") == 0) {
+   ret= libjte_set_checksum_iso(jte, arg);
+   if(ret <= 0)
+     goto jte_failed;
+ } else if(strcmp(aspect, "checksum_template") == 0 ||
+           strcmp(aspect, "-checksum_algorithm_template") == 0) {
+   ret= libjte_set_checksum_template(jte, arg);
+   if(ret <= 0)
+     goto jte_failed;
+ } else if(strcmp(aspect, "compression") == 0 ||
+           strcmp(aspect, "-jigdo-template-compress") == 0) {
+   ret= libjte_set_compression(jte, arg);
+   if(ret <= 0)
+     goto jte_failed;
+ } else if(strcmp(aspect, "exclude") == 0 ||
+           strcmp(aspect, "-jigdo-exclude") == 0) {
+   ret= libjte_add_exclude(jte, arg);
+   if(ret <= 0)
+     goto jte_failed;
+ } else if(strcmp(aspect, "demand_md5") == 0 ||
+           strcmp(aspect, "-jigdo-force-md5") == 0) {
+   ret= libjte_add_md5_demand(jte, arg);
+   if(ret <= 0)
+     goto jte_failed;
+ } else if(strcmp(aspect, "mapping") == 0 ||
+           strcmp(aspect, "-jigdo-map") == 0) {
+   ret= libjte_add_mapping(jte, arg);
+   if(ret <= 0)
+     goto jte_failed;
+ } else {
+   sprintf(xorriso->info_text, "-jigdo: unknown aspect '%s'", aspect); 
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+
+ ret= Xorriso_lst_new(&(xorriso->jigdo_params), aspect, xorriso->jigdo_params,
+                      1);
+ if(ret > 0)
+   ret= Xorriso_lst_new(&(xorriso->jigdo_values), arg, xorriso->jigdo_values,
+                        1);
+ if(ret <= 0) {
+   Xorriso_no_malloc_memory(xorriso, NULL, 0);
+   return(-1);
+ }
+ Xorriso_process_msg_queues(xorriso, 0);
+ return(1);
+
+bad_arg:
+ sprintf(xorriso->info_text, "-jigdo %s : unknown argument '%s'", aspect, arg);
+ Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+ return(0);
+
+jte_failed:
+ while(1) {
+   msg= libjte_get_next_message(xorriso->libjte_handle);
+   if(msg == NULL)
+ break;
+   sprintf(xorriso->info_text, "%1.4095s", msg);
+   free(msg);
+   msg= NULL;
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+ }
+ sprintf(xorriso->info_text, "Experienced libjte failure with: -jigdo %s %s",
+         aspect, arg);
+ Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+ return(0);
+
+#else /* Xorriso_with_libjtE */
+
+ sprintf(xorriso->info_text,
+         "Jigdo Template Extraction was not enabled at compile time");
+ Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+ return(0);
+
+#endif /* ! Xorriso_with_libjtE */
+
+}
+
