@@ -255,6 +255,55 @@ ex:;
 }
 
 
+/* Option -append_partition */
+int Xorriso_option_append_partition(struct XorrisO *xorriso, char *partno_text,
+                                   char *type_text, char *image_path, int flag)
+{
+ int partno = 0, type_code= -1, i;
+ unsigned int unum;
+ char *tpt;
+ static char *part_type_names[] = {"FAT12", "FAT16", "Linux", NULL};
+ static int part_type_codes[]   = {   0x01,    0x06,   0x83};
+
+ sscanf(partno_text, "%d", &partno);
+ if(partno < 1 || partno > 4) {
+   sprintf(xorriso->info_text,
+           "-append_partition:  Partition number '%s' is out of range (1...4)",
+           partno_text);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ for(i= 0; part_type_names[i] != NULL; i++)
+   if(strcmp(part_type_names[i], type_text) == 0)
+ break;
+ if(part_type_names[i] != NULL)
+   type_code= part_type_codes[i];
+ if(type_code < 0) {
+   tpt= type_text;
+   if(strncmp(tpt, "0x", 2) == 0)
+     tpt+= 2;
+   else
+     goto bad_type;
+   unum= 0xffffffff;
+   sscanf(tpt, "%X", &unum);
+   if(unum > 0xff) {
+bad_type:;
+     sprintf(xorriso->info_text,
+       "-append_partition: Partition type '%s' is out of range (0x00...0xff)",
+       type_text);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     return(0);
+   }
+   type_code= unum;
+ }
+
+ if(Sfile_str(xorriso->appended_partitions[partno - 1], image_path, 0) <= 0)
+   return(-1);
+ xorriso->appended_part_types[partno - 1]= type_code;
+ return(1);
+}
+
+
 /* Option -application_id */
 int Xorriso_option_application_id(struct XorrisO *xorriso, char *name,
                                   int flag)
@@ -567,7 +616,7 @@ int Xorriso_option_boot_image(struct XorrisO *xorriso, char *form,
 {
  int was_ok= 1, ret, isolinux_grub= 0, count;
  unsigned int u;
- char *formpt, *treatpt;
+ char *formpt, *treatpt, eff_path[SfileadrL];
  double num;
 
  formpt= form;
@@ -614,6 +663,8 @@ treatment_patch:;
    xorriso->patch_isolinux_image= 0;
    xorriso->boot_image_bin_path[0]= 0;
    xorriso->patch_system_area= 0;
+   if((xorriso->system_area_options & 0xfc ) == 0)
+     xorriso->system_area_options= 0; /* Reset eventual type 0 flags */
    if(xorriso->boot_count > 0) {
      ret= Xorriso_attach_boot_image(xorriso, 2); /* dispose boot images */
      if(ret <= 0)
@@ -730,6 +781,37 @@ treatment_patch:;
    xorriso->keep_boot_image= 0;
    xorriso->boot_efi_default= 1;
 
+ } else if(strncmp(treatpt, "mips_path=", 10) == 0) {
+   sprintf(eff_path, "-boot_image %s mips_path=", formpt);
+   ret= Xorriso_coordinate_system_area(xorriso, 1, 0, eff_path, 0);
+   if(ret <= 0)
+     return(ret);
+   ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, treatpt + 10,
+                                   eff_path, 2);
+   if(ret <= 0)
+     return(ret);
+   ret= Xorriso_add_mips_boot_file(xorriso, eff_path, 0);
+   if(ret <= 0)
+     return(ret);
+
+ } else if(strncmp(treatpt, "mipsel_path=", 12) == 0) {
+   sprintf(eff_path, "-boot_image %s mipsel_path=", formpt);
+   ret= Xorriso_coordinate_system_area(xorriso, 2, 0, eff_path, 0);
+   if(ret <= 0)
+     return(ret);
+   ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, treatpt + 12,
+                                   eff_path, 2);
+   if(ret <= 0)
+     return(ret);
+   ret= Xorriso_add_mips_boot_file(xorriso, eff_path, 2);
+   if(ret <= 0)
+     return(ret);
+
+ } else if(strcmp(treatpt, "mips_discard") == 0 ||
+           strcmp(treatpt, "mipsel_discard") == 0) {
+   xorriso->system_area_options&= ~0xfc; /* system area type 0 */
+   Xorriso_add_mips_boot_file(xorriso, "", 1); /* give up MIPS boot files */
+
  } else if(strncmp(treatpt, "boot_info_table=", 16)==0) {
    if(strcmp(treatpt + 16, "off") == 0)
      xorriso->patch_isolinux_image= 0;
@@ -771,23 +853,27 @@ treatment_patch:;
    }
 
  } else if(strncmp(treatpt, "system_area=", 12) == 0) {
+   if(strcmp(formpt, "isolinux")==0) {
+     ret= Xorriso_coordinate_system_area(xorriso, 0, 2,
+                                       "-boot_image isolinux system_area=", 0);
+     if(ret <= 0)
+       return(ret);
+   }
    ret= Xorriso_set_system_area_path(xorriso, treatpt + 12, 0);
    if(ret <= 0)
      return(ret);
-   if(strcmp(formpt, "isolinux")==0)
-     xorriso->system_area_options= (xorriso->system_area_options & ~3) | 2;
 
  } else if(strncmp(treatpt, "partition_table=", 16)==0) {
    if(strcmp(treatpt + 16, "off") == 0) {
      xorriso->system_area_options&= ~3;
-     xorriso->patch_system_area= xorriso->system_area_options;
    } else if(strcmp(treatpt + 16, "on") == 0) {
-     xorriso->system_area_options&= ~3;
+     sprintf(eff_path, "-boot_image %s partition_table=", formpt);
      if(strcmp(formpt, "isolinux")==0)
-       xorriso->system_area_options|= 2;
+       ret= Xorriso_coordinate_system_area(xorriso, 0, 2, eff_path, 0);
      else
-       xorriso->system_area_options|= 1;
-     xorriso->patch_system_area= xorriso->system_area_options;
+       ret= Xorriso_coordinate_system_area(xorriso, 0, 1, eff_path, 0);
+     if(ret <= 0)
+       return(ret);
    } else
      was_ok= 0;
 
