@@ -1376,6 +1376,13 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Specifies the name of the Abstract File. (37 chars)",
 "  -joliet \"on\"|\"off\"",
 "                 Generate Joliet info additional to Rock Ridge info.",
+"  -jigdo \"clear\"|\"template_path\"|\"jigdo_path\"|\"md5_path\"",
+"         |\"min_size\"|\"checksum_iso\"|\"checksum_template\"",
+"         |\"compression\"|\"exclude\"|\"demand_md5\"|\"mapping\"",
+"         |\"checksum_iso\"|\"checksum_template\"",
+"         value",
+"                 Clear Jigdo Template Extraction parameter list or add a",
+"                 parameter with its value to that list.",
 "  -compliance rule[:rule...]",
 "                 Allow more or less harmless deviations from strict standards",
 "                 compliance.",
@@ -1383,6 +1390,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              |\"discard\"|\"keep\"|\"patch\"|\"dir=\"|\"bin_path=\"",
 "              |\"cat_path=\"|\"cat_hidden=on|iso_rr|joliet|off\"",
 "              |\"load_size=\"|\"system_area=\"|\"partition_table=on|off\"",
+"              |\"mips_path=\"|\"mipsel_path=\"|\"mips_discard\"",
+"              |\"show_status\"",
 "                 Whether to discard or keep an exiting El Torito boot image.",
 "                 ISOLINUX can be made bootable by dir=/ or dir=/isolinux",
 "                 or dir=/boot/isolinux. Others, like GRUB, by bin_path=...",
@@ -1391,6 +1400,12 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 ISO image by the usual commands like -map or -add.",
 "                 system_area= and partition_table= are for MBR based booting",
 "                 from USB stick. The system_area= file needs not to be added.",
+"                 mips_path= adds Big Endian MIPS boot files. mipsel_path=",
+"                 sets one Little Endian MIPS boot file. Both are mutually",
+"                 exclusive and exclusive to MBR production.",
+"  -append_partition partition_number type_code disk_path",
+"                 Append a prepared filesystem image after the end of the",
+"                 ISO image. Caution: Will be overwritten by multi-session.",
 "",
 "  -uid uid       User id to be used for the whole multi-session ISO image.",
 "  -gid gid       Group id for the same purpose.",
