@@ -47,6 +47,17 @@ int Xorriso_option_iso_rr_pattern(struct XorrisO *xorriso, char *mode,int flag)
 }
 
 
+/* Option -jigdo aspect argument */
+int Xorriso_option_jigdo(struct XorrisO *xorriso, char *aspect, char *arg,
+                         int flag)
+{
+ int ret;
+
+ ret= Xorriso_jigdo_interpreter(xorriso, aspect, arg, 0);
+ return(ret);
+}
+
+
 /* Option -joliet "on"|"off" */
 int Xorriso_option_joliet(struct XorrisO *xorriso, char *mode, int flag)
 {
