@@ -478,11 +478,15 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
  static char arg2_commands[][40]= {
     "assert_volid","boot_image","compare","compare_r","drive_class",
     "errfile_log","error_behavior","extract","extract_single",
-    "load","logfile",
+    "jigdo","load","logfile",
     "map","map_single","page","return_with",
     "scdbackup_tag","update","update_r","volume_date",
     ""
  };
+ static char arg3_commands[][40]= {
+    "append_partition",
+    ""
+ };
  static char arg4_commands[][40]= {
     "cut_out","extract_cut","mount","mount_cmd","paste_in","session_string",
     ""
@@ -530,6 +534,10 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
  for(i=0; arg2_commands[i][0]!=0; i++)
    if(strcmp(arg2_commands[i], cmd)==0)
      return(1);
+ *count= 3;
+ for(i=0; arg3_commands[i][0]!=0; i++)
+   if(strcmp(arg3_commands[i], cmd)==0)
+     return(1);
  *count= 4;
  for(i=0; arg4_commands[i][0]!=0; i++)
    if(strcmp(arg4_commands[i], cmd)==0)
@@ -648,6 +656,11 @@ next_command:;
    ret= Xorriso_option_alter_date(xorriso, arg1, arg2, argc, argv, idx,
                                   strlen(cmd)>10);
 
+ } else if(strcmp(cmd,"append_partition")==0) {
+   (*idx)+= 3;
+   ret= Xorriso_option_append_partition(xorriso, arg1, arg2, argv[*idx - 1],
+                                        0);
+
  } else if(strcmp(cmd,"application_id")==0) {
    (*idx)++;
    ret= Xorriso_option_application_id(xorriso, arg1, 0);
@@ -948,6 +961,10 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_iso_rr_pattern(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"jigdo")==0) {
+   (*idx)+= 2;
+   ret= Xorriso_option_jigdo(xorriso, arg1, arg2, 0);
+
  } else if(strcmp(cmd,"joliet")==0) {
    (*idx)++;
    ret= Xorriso_option_joliet(xorriso, arg1, 0);
