@@ -1091,7 +1091,7 @@ int Xorriso_status(struct XorrisO *xorriso, char *filter, FILE *fp, int flag)
  static char channel_prefixes[4][4]= {".","R","I","M"};
  static char load_names[][20]= {"auto", "session", "track", "lba", "volid"};
  static int max_load_mode= 4;
- struct Xorriso_lsT *paths, *leafs, *s;
+ struct Xorriso_lsT *paths, *leafs, *s, *plst, *vlst;
 
  no_defaults= flag&1;
  line= xorriso->result_line;
@@ -1484,6 +1484,22 @@ int Xorriso_status(struct XorrisO *xorriso, char *filter, FILE *fp, int flag)
  if(!(is_default && no_defaults))
    Xorriso_status_result(xorriso,filter,fp,flag&2);
 
+ Xorriso_lst_get_last(xorriso->jigdo_params, &plst, 0);
+ Xorriso_lst_get_last(xorriso->jigdo_values, &vlst, 0);
+ if(plst == NULL || vlst == NULL) {
+   is_default= 1;
+   sprintf(line,"-jigdo clear 'all'\n");
+   if(!(is_default && no_defaults))
+     Xorriso_status_result(xorriso, filter, fp, flag & 2);
+ }
+ while(plst != NULL && vlst != NULL) {
+   sprintf(line,"-jigdo %s %s\n", Xorriso_lst_get_text(plst, 0), 
+           Text_shellsafe(Xorriso_lst_get_text(vlst, 0), sfe, 0));
+   Xorriso_status_result(xorriso, filter, fp, flag & 2);
+   plst= Xorriso_lst_get_prev(plst, 0);
+   vlst= Xorriso_lst_get_prev(vlst, 0);
+ }
+
  if(xorriso->do_global_uid) {
    sprintf(line,"-uid %lu\n", (unsigned long) xorriso->global_uid);
    Xorriso_status_result(xorriso,filter,fp,flag&2);
