@@ -25,6 +25,21 @@
 #include <errno.h>
 
 #include <fcntl.h>
+#include <stdint.h>
+
+#ifdef Xorriso_standalonE
+
+#ifdef Xorriso_with_libjtE
+#include "../libjte/libjte.h"
+#endif
+
+#else
+
+#ifdef Xorriso_with_libjtE
+#include <libjte/libjte.h>
+#endif
+
+#endif /* ! Xorriso_standalonE */
 
 #include "xorriso.h"
 #include "xorriso_private.h"
@@ -117,7 +132,7 @@ no_track:;
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
    }
  }
- if(xorriso->alignment == 0) {
+ if(xorriso->alignment == 0 && ! (xorriso->no_emul_toc & 1)) {
    ret= isoburn_needs_emulation(drive);
    if(ret > 0) {
      /* Take care that the session is padded up to the future NWA.
@@ -138,6 +153,12 @@ no_track:;
    } else if(padding < xorriso->alignment)
      padding= xorriso->alignment;
  }
+
+#ifdef Xorriso_with_libjtE
+ if(xorriso->libjte_handle != NULL)
+   padding= 0;  /* JTE : no multi-session, no_emul_toc, padding in libisofs */
+#endif /* ! Xorriso_with_libjtE */
+
  burn_track_define_data(tracks[0], 0, padding * 2048, 0, BURN_MODE1);
  Xorriso_process_msg_queues(xorriso,0);
 
@@ -715,6 +736,12 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
  isoburn_igopt_set_scdbackup_tag(sopts, xorriso->scdbackup_tag_name,
                                  xorriso->scdbackup_tag_time,
                                  xorriso->scdbackup_tag_written);
+ for(i= 0; i < 4; i++) {
+   if(xorriso->appended_partitions[i][0] == 0)
+ continue;
+   ret= isoburn_igopt_set_partition_img(sopts, i + 1,
+             xorriso->appended_part_types[i], xorriso->appended_partitions[i]);
+ }
  if(image!=NULL && 12+strlen(Xorriso_timestamP)<80) {
    strcpy(xorriso_id, xorriso->preparer_id);
    img_id= (char *) iso_image_get_data_preparer_id(image);
@@ -733,6 +760,23 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
                     xorriso->vol_expiration_time, xorriso->vol_effective_time,
                     xorriso->vol_uuid);
 
+#ifdef Xorriso_with_libjtE
+ if(xorriso->libjte_handle) {
+   ret= libjte_set_outfile(xorriso->libjte_handle, xorriso->outdev);
+   Xorriso_process_msg_queues(xorriso, 0);
+   if(ret <= 0)
+     goto ex;
+
+   /* >>> Check whether the mandatory parameters are set */;
+
+   isoburn_igopt_attach_jte(sopts, xorriso->libjte_handle);
+
+   /* Padding to be done by libisofs, not by libburn */
+   isoburn_igopt_set_tail_blocks(sopts, (uint32_t) (xorriso->padding / 2048));
+ }
+#endif /* Xorriso_with_libjtE */
+
+
  /* Make final abort check before starting expensive activities */
  ret= Xorriso_eval_problem_status(xorriso, 1, 0);
  if(ret<0)
@@ -943,6 +987,8 @@ int Xorriso_pacifier_loop(struct XorrisO *xorriso, struct burn_drive *drive,
                           int flag)
 {
  int ret, size, free_bytes, i, aborting= 0, emul, buffer_fill= 50, last_sector;
+ int iso_wait_counter= 0, iso_cancel_limit= 5, iso_break_limit= 20;
+ int iso_cancelled= 0;
  struct burn_progress progress;
  char *status_text, date_text[80], *speed_unit, mem_text[8];
  enum burn_drive_status drive_status;
@@ -951,6 +997,9 @@ int Xorriso_pacifier_loop(struct XorrisO *xorriso, struct burn_drive *drive,
  double first_base_count= 0.0, norm= 0.0, now_time, fract_offset= 0.0;
  double measured_speed, speed_factor= 1385000, quot;
  time_t time_prediction;
+ IsoImage *image= NULL;
+
+ image= isoburn_get_attached_image(drive);
 
  start_time= Sfile_microtime(0);
  while(burn_drive_get_status(drive, NULL) == BURN_DRIVE_SPAWNING)
@@ -975,8 +1024,33 @@ int Xorriso_pacifier_loop(struct XorrisO *xorriso, struct burn_drive *drive,
    last_time= current_time;
    last_sector= progress.sector;
    drive_status= burn_drive_get_status(drive, &progress);
-   if(drive_status == BURN_DRIVE_IDLE)
+
+   if(drive_status == BURN_DRIVE_IDLE) {
+
+     /* To avoid a race condition between burn_source and libisofs
+        writer thread: Wait for ISO generator thread to have finished.
+     */
+     if(image == NULL)
  break;
+     if(!iso_image_generator_is_running(image))
+ break;
+     iso_wait_counter++;
+     if(iso_wait_counter > iso_cancel_limit && !iso_cancelled) {
+       isoburn_cancel_prepared_write(drive, NULL, 0);
+       iso_cancelled= 1;
+     } else if(iso_wait_counter > iso_break_limit) {
+
+       /* >>> This can be dangerous. xorriso will free memory objects which
+              might be in use in the unstoppable libisofs thread.
+              The current implementation of ecma119.c:bs_cancel() seems to
+              wait for the writer thread to end. Probably one should make
+              this a part of the libisofs API specs and break the loop
+              directly after isoburn_cancel_prepared_write().
+       */
+
+ break;
+     }
+   }
    current_time= Sfile_microtime(0);
    if(drive_status == BURN_DRIVE_WRITING && progress.sectors > 0) {
      if(current_time-last_time>0.2)
