@@ -61,7 +61,7 @@ struct XorrisO;
 */
 #define Xorriso_header_version_majoR  0
 #define Xorriso_header_version_minoR  6
-#define Xorriso_header_version_micrO  2
+#define Xorriso_header_version_micrO  4
 
 
 /** Eventually something like ".pl01" to indicate a bug fix. Normally empty.
@@ -562,15 +562,15 @@ int Xorriso_lst_destroy_all(struct Xorriso_lsT **lstring, int flag);
    Xorriso_eval_problem_status()
    One should follow its eventual advice to abort.
 
-   Options with a varying number of arguments get them passed like
+   Options with a varying number of arguments get then passed like
    Xorriso_interpreter(). E.g.:
      int Xorriso_option_add(struct XorrisO *xorriso, int argc, char **argv,
                             int *idx, int flag);
-   They begin to read the arguments at position *idx and will see the list
-   end either at the next argv which contains the -list_delimiter text or
-   at argv[argc-1]. After the call, *idx will be the index of the first not
-   yet interpreted argv.
-
+   The option functions will begin to read the arguments at position *idx
+   and will see the list end either at the next argv which contains the
+   -list_delimiter text or at argv[argc-1].
+   After the call, *idx will be the index of the first not yet interpreted
+   argv.
 */
 
 
@@ -603,6 +603,10 @@ int Xorriso_option_alter_date(struct XorrisO *xorriso,
                               char *time_type, char *timestring,
                               int argc, char **argv, int *idx, int flag);
 
+/* Option -append_partition */
+int Xorriso_option_append_partition(struct XorrisO *xorriso, char *partno_text,
+                                  char *type_text, char *image_path, int flag);
+
 /* Option -application_id */
 int Xorriso_option_application_id(struct XorrisO *xorriso, char *name, 
                                   int flag);
@@ -854,6 +858,10 @@ int Xorriso_option_history(struct XorrisO *xorriso, char *line, int flag);
 int Xorriso_option_iso_rr_pattern(struct XorrisO *xorriso, char *mode,
                                   int flag);
 
+/* Option -jigdo aspect argument */
+int Xorriso_option_jigdo(struct XorrisO *xorriso, char *aspect, char *arg,
+                         int flag);
+
 /* Option -joliet "on"|"off" */
 int Xorriso_option_joliet(struct XorrisO *xorriso, char *mode, int flag);
 
