@@ -300,6 +300,11 @@ struct XorrisO { /* the global context of xorriso */
                                                (a simple partition table)
                                          bit1= isohybrid boot image pointer
                                                and partition table
+                                         bit2-7= System area type
+                                               0= with bit0 or bit1: MBR
+                                                  else: unspecified type 
+                                               1= MIPS Big Endian Volume Header
+                                               2= MIPS Little Endian Boot Block
                                        */
  int patch_system_area;               /* Bits as of system_area_options.
                                          to be applied to the loaded system
@@ -317,6 +322,10 @@ struct XorrisO { /* the global context of xorriso */
  /* 1 to 255, 0= disabled/default */
  int partition_heads_per_cyl;
 
+ /* Path and type of image files to be appended as MBR partitions */
+ char appended_partitions[4][SfileadrL];
+ uint8_t appended_part_types[4];
+
  /* User settable PVD time stamps */
  time_t vol_creation_time;
  time_t vol_modification_time;
@@ -326,6 +335,16 @@ struct XorrisO { /* the global context of xorriso */
     and timezone 0 */
  char vol_uuid[17];
 
+#ifdef Xorriso_with_libjtE
+ /* Parameters and state of Jigdo Template Export environment */
+ struct libjte_env *libjte_handle;
+#endif
+
+ /* List of -jigdo parameters since the most recent -jigdo clear */
+ struct Xorriso_lsT *jigdo_params;
+ struct Xorriso_lsT *jigdo_values;
+
+
  /* LBA of boot image after image loading */
  int loaded_boot_bin_lba;
  /* Path of the catalog node after image loading */
