@@ -19,7 +19,7 @@
 */
 #define xorriso_libisoburn_req_major  0
 #define xorriso_libisoburn_req_minor  6
-#define xorriso_libisoburn_req_micro  2
+#define xorriso_libisoburn_req_micro  4
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
@@ -512,12 +512,21 @@ int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag);
 int Xorriso_boot_image_status(struct XorrisO *xorriso, char *filter, FILE *fp,
                               int flag);
 
+int Xorriso_add_mips_boot_file(struct XorrisO *xorriso, char *path, int flag);
+
+int Xorriso_coordinate_system_area(struct XorrisO *xorriso, int sa_type,
+                                   int options, char *cmd, int flag);
+
+
+
+
 /* A pseudo file type for El-Torito bootsectors as in man 2 stat :
    For now take the highest possible value.
 */
 #define Xorriso_IFBOOT S_IFMT
 
 
+
 int Exprtest_match(struct XorrisO *xorriso, struct ExprtesT *ftest,
                    void *node_pt, char *name, char *path,
                    struct stat *boss_stbuf, struct stat *stbuf, int flag);
@@ -540,5 +549,8 @@ int Xorriso_set_hidden(struct XorrisO *xorriso, void *in_node, char *path,
 int Xorriso_preparer_string(struct XorrisO *xorriso, char xorriso_id[129],
                             int flag);
 
+int Xorriso_jigdo_interpreter(struct XorrisO *xorriso, char *aspect, char *arg,
+                              int flag);
+
 #endif /* Xorrisoburn_includeD */
 
